@@ -253,7 +253,7 @@ append_section (GtkBuilder         *builder,
       is_new = TRUE;
     }
 
-  for (i = 0; keys_list[i].name != NULL; i++)
+  for (i = 0; keys_list != NULL && keys_list[i].name != NULL; i++)
     {
       CcKeyboardItem *item;
       gboolean ret;
@@ -300,7 +300,7 @@ append_section (GtkBuilder         *builder,
     }
 
   /* Add the keys to the hash table */
-  if (is_new && keys_array->len > 0)
+  if (is_new)
     {
       static gboolean have_sep = FALSE;
 
@@ -649,6 +649,10 @@ append_sections_from_gconf (GtkBuilder *builder, const gchar *gconf_path)
           g_free (keys[i].name);
         }
     }
+  else
+    {
+      append_section (builder, _("Custom Shortcuts"), BINDING_GROUP_USER, NULL);
+    }
 
   g_array_free (entries, TRUE);
 }
@@ -924,6 +928,7 @@ remove_custom_shortcut (GtkTreeModel *model, GtkTreeIter *iter)
   gchar *base;
   CcKeyboardItem *item;
   GPtrArray *keys_array;
+  GError *err = NULL;
 
   gtk_tree_model_get (model, iter,
                       DETAIL_KEYENTRY_COLUMN, &item,
@@ -934,10 +939,15 @@ remove_custom_shortcut (GtkTreeModel *model, GtkTreeIter *iter)
 
   client = gconf_client_get_default ();
 
-  base = g_path_get_dirname (item->gconf_key_dir);
+  base = g_strdup (item->gconf_key_dir);
   g_object_unref (item);
 
-  gconf_client_recursive_unset (client, base, 0, NULL);
+  if (gconf_client_recursive_unset (client, base, 0, &err) == FALSE)
+    {
+      g_warning ("Failed to unset GConf directory '%s': %s", base, err->message);
+      g_error_free (err);
+    }
+
   g_free (base);
   /* suggest sync now so the unset directory actually gets dropped;
    * if we don't do this we may end up with 'zombie' shortcuts when
