@@ -50,6 +50,13 @@ G_DEFINE_DYNAMIC_TYPE (CcPrintersPanel, cc_printers_panel, CC_TYPE_PANEL)
 #define CLOCK_SCHEMA "org.gnome.desktop.interface"
 #define CLOCK_FORMAT_KEY "clock-format"
 
+#define RENEW_INTERVAL        500
+#define SUBSCRIPTION_DURATION 600
+
+#define CUPS_DBUS_NAME      "org.cups.cupsd.Notifier"
+#define CUPS_DBUS_PATH      "/org/cups/cupsd/Notifier"
+#define CUPS_DBUS_INTERFACE "org.cups.cupsd.Notifier"
+
 struct _CcPrintersPanelPrivate
 {
   GtkBuilder *builder;
@@ -58,6 +65,7 @@ struct _CcPrintersPanelPrivate
 
   cups_dest_t *dests;
   gchar **dest_model_names;
+  gchar **ppd_file_names;
   int num_dests;
   int current_dest;
 
@@ -77,6 +85,10 @@ struct _CcPrintersPanelPrivate
 
   PpNewPrinterDialog *pp_new_printer_dialog;
 
+  GDBusProxy      *cups_proxy;
+  GDBusConnection *cups_bus_connection;
+  gint             subscription_id;
+
   gpointer dummy;
 };
 
@@ -86,6 +98,8 @@ static void actualize_allowed_users_list (CcPrintersPanel *self);
 static void actualize_sensitivity (gpointer user_data);
 static void printer_disable_cb (GObject *gobject, GParamSpec *pspec, gpointer user_data);
 static void printer_set_default_cb (GtkToggleButton *button, gpointer user_data);
+static void detach_from_cups_notifier (gpointer data);
+static void free_dests (CcPrintersPanel *self);
 
 static void
 cc_printers_panel_get_property (GObject    *object,
@@ -119,16 +133,7 @@ cc_printers_panel_dispose (GObject *object)
   CcPrintersPanelPrivate *priv = CC_PRINTERS_PANEL (object)->priv;
   int                     i;
 
-  if (priv->num_dests > 0)
-    {
-      for (i = 0; i < priv->num_dests; i++)
-        g_free (priv->dest_model_names[i]);
-      g_free (priv->dest_model_names);
-      cupsFreeDests (priv->num_dests, priv->dests);
-    }
-  priv->dests = NULL;
-  priv->num_dests = 0;
-  priv->current_dest = -1;
+  free_dests (CC_PRINTERS_PANEL (object));
 
   if (priv->num_jobs > 0)
     cupsFreeJobs (priv->num_jobs, priv->jobs);
@@ -158,6 +163,8 @@ cc_printers_panel_dispose (GObject *object)
       priv->lockdown_settings = NULL;
     }
 
+  detach_from_cups_notifier (CC_PRINTERS_PANEL (object));
+
   G_OBJECT_CLASS (cc_printers_panel_parent_class)->dispose (object);
 }
 
@@ -185,6 +192,228 @@ cc_printers_panel_class_finalize (CcPrintersPanelClass *klass)
 {
 }
 
+static void
+on_cups_notification (GDBusConnection *connection,
+                      const char      *sender_name,
+                      const char      *object_path,
+                      const char      *interface_name,
+                      const char      *signal_name,
+                      GVariant        *parameters,
+                      gpointer         user_data)
+{
+  CcPrintersPanel        *self = (CcPrintersPanel*) user_data;
+  CcPrintersPanelPrivate *priv;
+  gboolean                printer_is_accepting_jobs;
+  gchar                  *printer_name = NULL;
+  gchar                  *text = NULL;
+  gchar                  *printer_uri = NULL;
+  gchar                  *printer_state_reasons = NULL;
+  gchar                  *job_state_reasons = NULL;
+  gchar                  *job_name = NULL;
+  guint                   job_id;
+  gint                    printer_state;
+  gint                    job_state;
+  gint                    job_impressions_completed;
+  static const char * const requested_attrs[] = {
+    "job-printer-uri",
+    "job-originating-user-name"};
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  if (g_strcmp0 (signal_name, "PrinterAdded") != 0 &&
+      g_strcmp0 (signal_name, "PrinterDeleted") != 0 &&
+      g_strcmp0 (signal_name, "PrinterStateChanged") != 0 &&
+      g_strcmp0 (signal_name, "JobCreated") != 0 &&
+      g_strcmp0 (signal_name, "JobCompleted") != 0)
+    return;
+
+  if (g_variant_n_children (parameters) == 1)
+    g_variant_get (parameters, "(&s)", &text);
+  else if (g_variant_n_children (parameters) == 6)
+    {
+      g_variant_get (parameters, "(&s&s&su&sb)",
+                     &text,
+                     &printer_uri,
+                     &printer_name,
+                     &printer_state,
+                     &printer_state_reasons,
+                     &printer_is_accepting_jobs);
+    }
+  else if (g_variant_n_children (parameters) == 11)
+    {
+      g_variant_get (parameters, "(&s&s&su&sbuu&s&su)",
+                     &text,
+                     &printer_uri,
+                     &printer_name,
+                     &printer_state,
+                     &printer_state_reasons,
+                     &printer_is_accepting_jobs,
+                     &job_id,
+                     &job_state,
+                     &job_state_reasons,
+                     &job_name,
+                     &job_impressions_completed);
+    }
+
+  if (g_strcmp0 (signal_name, "PrinterAdded") == 0 ||
+      g_strcmp0 (signal_name, "PrinterDeleted") == 0 ||
+      g_strcmp0 (signal_name, "PrinterStateChanged") == 0)
+    actualize_printers_list (self);
+  else if (g_strcmp0 (signal_name, "JobCreated") == 0 ||
+           g_strcmp0 (signal_name, "JobCompleted") == 0)
+    {
+      http_t *http;
+      gchar  *job_uri;
+      ipp_t  *request, *response;
+
+      job_uri = g_strdup_printf ("ipp://localhost/jobs/%d", job_id);
+      if ((http = httpConnectEncrypt (cupsServer (), ippPort (),
+                                     cupsEncryption ())) != NULL)
+        {
+          request = ippNewRequest(IPP_GET_JOB_ATTRIBUTES);
+          ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
+                       "job-uri", NULL, job_uri);
+          ippAddStrings (request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
+                         "requested-attributes", G_N_ELEMENTS (requested_attrs), NULL, requested_attrs);
+          response = cupsDoRequest(http, request, "/");
+
+          if (response)
+            {
+              if (response->request.status.status_code <= IPP_OK_CONFLICT)
+                {
+                  ipp_attribute_t *attr_username = NULL;
+                  ipp_attribute_t *attr_printer_uri = NULL;
+
+                  attr_username = ippFindAttribute(response, "job-originating-user-name", IPP_TAG_NAME);
+                  attr_printer_uri = ippFindAttribute(response, "job-printer-uri", IPP_TAG_URI);
+                  if (attr_username && attr_printer_uri &&
+                      g_strcmp0 (attr_username->values[0].string.text, cupsUser ()) == 0 &&
+                      g_strrstr (attr_printer_uri->values[0].string.text, "/") != 0 &&
+                      priv->current_dest >= 0 &&
+                      priv->current_dest < priv->num_dests &&
+                      priv->dests != NULL &&
+                      g_strcmp0 (g_strrstr (attr_printer_uri->values[0].string.text, "/") + 1,
+                                 priv->dests[priv->current_dest].name) == 0)
+                    actualize_jobs_list (self);
+                }
+              ippDelete(response);
+            }
+          httpClose (http);
+        }
+      g_free (job_uri);
+    }
+}
+
+static gboolean
+renew_subscription (gpointer data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) data;
+  static const char * const events[] = {
+          "printer-added",
+          "printer-deleted",
+          "printer-state-changed",
+          "job-created",
+          "job-completed"};
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  priv->subscription_id = renew_cups_subscription (priv->subscription_id,
+                                                   events,
+                                                   G_N_ELEMENTS (events),
+                                                   SUBSCRIPTION_DURATION);
+
+  return TRUE;
+}
+
+static void
+attach_to_cups_notifier (gpointer data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) data;
+  GError                 *error = NULL;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  renew_subscription (self);
+  g_timeout_add_seconds (RENEW_INTERVAL, renew_subscription, self);
+
+  error = NULL;
+  priv->cups_proxy = g_dbus_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                                    0,
+                                                    NULL,
+                                                    CUPS_DBUS_NAME,
+                                                    CUPS_DBUS_PATH,
+                                                    CUPS_DBUS_INTERFACE,
+                                                    NULL,
+                                                    &error);
+
+  if (error)
+    {
+      g_warning ("%s", error->message);
+      return;
+    }
+
+  priv->cups_bus_connection = g_dbus_proxy_get_connection (priv->cups_proxy);
+
+  g_dbus_connection_signal_subscribe (priv->cups_bus_connection,
+                                      NULL,
+                                      CUPS_DBUS_INTERFACE,
+                                      NULL,
+                                      CUPS_DBUS_PATH,
+                                      NULL,
+                                      0,
+                                      on_cups_notification,
+                                      self,
+                                      NULL);
+}
+
+static void
+detach_from_cups_notifier (gpointer data)
+{
+  CcPrintersPanelPrivate *priv;
+  CcPrintersPanel        *self = (CcPrintersPanel*) data;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  cancel_cups_subscription (priv->subscription_id);
+  priv->subscription_id = -1;
+
+  if (priv->cups_proxy != NULL) {
+    g_object_unref (priv->cups_proxy);
+    priv->cups_proxy = NULL;
+  }
+}
+
+static void
+free_dests (CcPrintersPanel *self)
+{
+  CcPrintersPanelPrivate *priv;
+  gint                    i;
+
+  priv = PRINTERS_PANEL_PRIVATE (self);
+
+  if (priv->num_dests > 0)
+    {
+      for (i = 0; i < priv->num_dests; i++)
+        {
+          g_free (priv->dest_model_names[i]);
+          if (priv->ppd_file_names[i]) {
+            g_unlink (priv->ppd_file_names[i]);
+            g_free (priv->ppd_file_names[i]);
+          }
+        }
+      g_free (priv->dest_model_names);
+      g_free (priv->ppd_file_names);
+      cupsFreeDests (priv->num_dests, priv->dests);
+    }
+  priv->dests = NULL;
+  priv->num_dests = 0;
+  priv->current_dest = -1;
+  priv->dest_model_names = NULL;
+  priv->ppd_file_names = NULL;
+}
+
 enum
 {
   NOTEBOOK_INFO_PAGE = 0,
@@ -304,7 +533,30 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
 
   priv->current_dest = id;
 
-  actualize_jobs_list (self);
+  if (!(priv->current_dest >= 0 &&
+        priv->current_dest < priv->num_dests &&
+        priv->dests != NULL &&
+        priv->current_job >= 0 &&
+        priv->current_job < priv->num_jobs &&
+        priv->jobs != NULL &&
+        g_strcmp0 (priv->dests[priv->current_dest].name,
+                   priv->jobs[priv->current_job].dest) == 0))
+    {
+      actualize_jobs_list (self);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "job-release-button");
+      gtk_widget_set_sensitive (widget, FALSE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "job-hold-button");
+      gtk_widget_set_sensitive (widget, FALSE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "job-cancel-button");
+      gtk_widget_set_sensitive (widget, FALSE);
+    }
+
   actualize_allowed_users_list (self);
 
   if (priv->current_dest >= 0 &&
@@ -334,9 +586,14 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
             printer_type = priv->dests[priv->current_dest].options[i].value;
         }
 
+      if (priv->ppd_file_names[priv->current_dest] == NULL)
+        priv->ppd_file_names[priv->current_dest] =
+          g_strdup (cupsGetPPD (priv->dests[priv->current_dest].name));
+
       if (priv->dest_model_names[priv->current_dest] == NULL)
         priv->dest_model_names[priv->current_dest] =
-          get_ppd_attribute (priv->dests[priv->current_dest].name, "ModelName");
+          get_ppd_attribute (priv->ppd_file_names[priv->current_dest],
+                             "ModelName");
 
       printer_model = g_strdup (priv->dest_model_names[priv->current_dest]);
 
@@ -631,18 +888,6 @@ printer_selection_changed_cb (GtkTreeSelection *selection,
     }
 
   actualize_sensitivity (self);
-
-  widget = (GtkWidget*)
-    gtk_builder_get_object (priv->builder, "job-release-button");
-  gtk_widget_set_sensitive (widget, FALSE);
-
-  widget = (GtkWidget*)
-    gtk_builder_get_object (priv->builder, "job-hold-button");
-  gtk_widget_set_sensitive (widget, FALSE);
-
-  widget = (GtkWidget*)
-    gtk_builder_get_object (priv->builder, "job-cancel-button");
-  gtk_widget_set_sensitive (widget, FALSE);
 }
 
 static void
@@ -675,16 +920,10 @@ actualize_printers_list (CcPrintersPanel *self)
         current_printer_instance = g_strdup (priv->dests[priv->current_dest].instance);
     }
 
-  if (priv->num_dests > 0)
-    {
-      for (i = 0; i < priv->num_dests; i++)
-        g_free (priv->dest_model_names[i]);
-      g_free (priv->dest_model_names);
-      cupsFreeDests (priv->num_dests, priv->dests);
-    }
+  free_dests (self);
   priv->num_dests = cupsGetDests (&priv->dests);
   priv->dest_model_names = g_new0 (gchar *, priv->num_dests);
-  priv->current_dest = -1;
+  priv->ppd_file_names = g_new0 (gchar *, priv->num_dests);
 
   treeview = (GtkTreeView*)
     gtk_builder_get_object (priv->builder, "printers-treeview");
@@ -768,8 +1007,18 @@ actualize_printers_list (CcPrintersPanel *self)
       g_free (default_icon_name);
     }
 
+  g_signal_handlers_block_by_func (
+    G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview))),
+    printer_selection_changed_cb,
+    self);
+
   gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (store));
 
+  g_signal_handlers_unblock_by_func (
+    G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview))),
+    printer_selection_changed_cb,
+    self);
+
   if (current_dest >= 0)
     {
       priv->current_dest = current_dest;
@@ -943,7 +1192,10 @@ actualize_jobs_list (CcPrintersPanel *self)
   GtkTreeView            *treeview;
   GtkTreeIter             iter;
   GSettings              *settings;
-  int                     i;
+  GtkWidget              *widget;
+  gchar                  *active_jobs;
+  gint                    num_jobs;
+  gint                    i;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
@@ -959,7 +1211,24 @@ actualize_jobs_list (CcPrintersPanel *self)
   if (priv->current_dest >= 0 &&
       priv->current_dest < priv->num_dests &&
       priv->dests != NULL)
-    priv->num_jobs = cupsGetJobs (&priv->jobs, priv->dests[priv->current_dest].name, 1, CUPS_WHICHJOBS_ACTIVE);
+    {
+      priv->num_jobs = cupsGetJobs (&priv->jobs, priv->dests[priv->current_dest].name, 1, CUPS_WHICHJOBS_ACTIVE);
+
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "printer-jobs-label");
+
+      num_jobs = priv->num_jobs < 0 ? 0 : (guint) priv->num_jobs;
+      /* Translators: there is n active print jobs on this printer */
+      active_jobs = g_strdup_printf (ngettext ("%u active", "%u active", num_jobs), num_jobs);
+
+      if (active_jobs)
+        {
+          gtk_label_set_text (GTK_LABEL (widget), active_jobs);
+          g_free (active_jobs);
+        }
+      else
+        gtk_label_set_text (GTK_LABEL (widget), EMPTY_TEXT);
+    }
 
   store = gtk_list_store_new (JOB_N_COLUMNS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
@@ -1065,26 +1334,17 @@ job_selection_changed_cb (GtkTreeSelection *selection,
     {
       ipp_jstate_t job_state = priv->jobs[priv->current_job].state;
 
-      if (job_state == IPP_JOB_HELD)
-        {
-          widget = (GtkWidget*)
-            gtk_builder_get_object (priv->builder, "job-release-button");
-          gtk_widget_set_sensitive (widget, TRUE);
-        }
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "job-release-button");
+      gtk_widget_set_sensitive (widget, job_state == IPP_JOB_HELD);
 
-      if (job_state == IPP_JOB_PENDING)
-        {
-          widget = (GtkWidget*)
-            gtk_builder_get_object (priv->builder, "job-hold-button");
-          gtk_widget_set_sensitive (widget, TRUE);
-        }
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "job-hold-button");
+      gtk_widget_set_sensitive (widget, job_state == IPP_JOB_PENDING);
 
-      if (job_state < IPP_JOB_CANCELED)
-        {
-          widget = (GtkWidget*)
-            gtk_builder_get_object (priv->builder, "job-cancel-button");
-          gtk_widget_set_sensitive (widget, TRUE);
-        }
+      widget = (GtkWidget*)
+        gtk_builder_get_object (priv->builder, "job-cancel-button");
+      gtk_widget_set_sensitive (widget, job_state < IPP_JOB_CANCELED);
     }
 }
 
@@ -1769,6 +2029,7 @@ printer_set_default_cb (GtkToggleButton *button,
   GError                 *error = NULL;
   char                   *ret_error = NULL;
   char                   *name = NULL;
+  const char             *cups_server;
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
@@ -1779,35 +2040,54 @@ printer_set_default_cb (GtkToggleButton *button,
 
   if (name)
     {
-      proxy = get_dbus_proxy (MECHANISM_BUS,
-                              "/",
-                              MECHANISM_BUS,
-                              TRUE);
-
-      if (!proxy)
-        return;
-
-      dbus_g_proxy_call (proxy, "PrinterSetDefault", &error,
-                         G_TYPE_STRING, name,
-                         G_TYPE_INVALID,
-                         G_TYPE_STRING, &ret_error,
-                         G_TYPE_INVALID);
-
-      g_object_unref (proxy);
-
-      if (error || (ret_error && ret_error[0] != '\0'))
+      cups_server = cupsServer ();
+      if (g_ascii_strncasecmp (cups_server, "localhost", 9) == 0 ||
+          g_ascii_strncasecmp (cups_server, "127.0.0.1", 9) == 0 ||
+          g_ascii_strncasecmp (cups_server, "::1", 3) == 0 ||
+          cups_server[0] == '/')
         {
-          if (error)
-            g_warning ("%s", error->message);
+          /* Clean .cups/lpoptions before setting
+           * default printer on local CUPS server.
+           */
+          set_local_default_printer (NULL);
 
-          if (ret_error && ret_error[0] != '\0')
-            g_warning ("%s", ret_error);
+          proxy = get_dbus_proxy (MECHANISM_BUS,
+                                  "/",
+                                  MECHANISM_BUS,
+                                  TRUE);
 
+          if (!proxy)
+            return;
+
+          dbus_g_proxy_call (proxy, "PrinterSetDefault", &error,
+                             G_TYPE_STRING, name,
+                             G_TYPE_INVALID,
+                             G_TYPE_STRING, &ret_error,
+                             G_TYPE_INVALID);
+
+          g_object_unref (proxy);
+
+          if (error || (ret_error && ret_error[0] != '\0'))
+            {
+              if (error)
+                g_warning ("%s", error->message);
+
+              if (ret_error && ret_error[0] != '\0')
+                g_warning ("%s", ret_error);
+            }
+          else
+            actualize_printers_list (self);
+
+          g_clear_error (&error);
         }
       else
-        actualize_printers_list (self);
-
-      g_clear_error (&error);
+        /* Store default printer to .cups/lpoptions
+         * if we are connected to a remote CUPS server.
+         */
+        {
+          set_local_default_printer (name);
+          actualize_printers_list (self);
+        }
 
       g_signal_handlers_block_by_func (G_OBJECT (button), printer_set_default_cb, self);
       gtk_toggle_button_set_active (button, priv->dests[priv->current_dest].is_default);
@@ -2051,7 +2331,10 @@ actualize_sensitivity (gpointer user_data)
 
   priv = PRINTERS_PANEL_PRIVATE (self);
 
-  is_authorized = g_permission_get_allowed (G_PERMISSION (priv->permission)) &&
+  is_authorized =
+    priv->permission &&
+    g_permission_get_allowed (G_PERMISSION (priv->permission)) &&
+    priv->lockdown_settings &&
     !g_settings_get_boolean (priv->lockdown_settings, "disable-print-setup");
 
   printer_selected = priv->current_dest >= 0 &&
@@ -2192,6 +2475,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
   priv->builder = gtk_builder_new ();
   priv->dests = NULL;
   priv->dest_model_names = NULL;
+  priv->ppd_file_names = NULL;
   priv->num_dests = 0;
   priv->current_dest = -1;
 
@@ -2205,6 +2489,14 @@ cc_printers_panel_init (CcPrintersPanel *self)
 
   priv->pp_new_printer_dialog = NULL;
 
+  priv->subscription_id = -1;
+  priv->cups_proxy = NULL;
+  priv->cups_bus_connection = NULL;
+
+  priv->lock_button = NULL;
+  priv->permission = NULL;
+  priv->lockdown_settings = NULL;
+
   gtk_builder_add_objects_from_file (priv->builder,
                                      DATADIR"/printers.ui",
                                      objects, &error);
@@ -2287,10 +2579,11 @@ cc_printers_panel_init (CcPrintersPanel *self)
   g_signal_connect (widget, "clicked", G_CALLBACK (switch_to_options_cb), self);
 
   priv->lockdown_settings = g_settings_new ("org.gnome.desktop.lockdown");
-  g_signal_connect (priv->lockdown_settings,
-                    "changed",
-                    G_CALLBACK (on_lockdown_settings_changed),
-                    self);
+  if (priv->lockdown_settings)
+    g_signal_connect (priv->lockdown_settings,
+                      "changed",
+                      G_CALLBACK (on_lockdown_settings_changed),
+                      self);
 
 
   /* Set junctions */
@@ -2340,6 +2633,10 @@ cc_printers_panel_init (CcPrintersPanel *self)
       on_permission_changed (priv->permission, NULL, self);
       priv->lock_button = widget;
     }
+  else
+    g_warning ("Your system does not have the cups-pk-helper's policy \
+\"org.opensuse.cupspkhelper.mechanism.all-edit\" installed. \
+Please check your installation");
 
 
   gtk_style_context_get_background_color (gtk_widget_get_style_context (top_widget),
@@ -2349,6 +2646,7 @@ cc_printers_panel_init (CcPrintersPanel *self)
   populate_printers_list (self);
   populate_jobs_list (self);
   populate_allowed_users_list (self);
+  attach_to_cups_notifier (self);
 
   gtk_container_add (GTK_CONTAINER (self), top_widget);
   gtk_widget_show_all (GTK_WIDGET (self));
